==> kyc-sdk-0.1.0.tgz: package/src/clients/provider-client.ts <==
import { PublicKey, SystemProgram, LAMPORTS_PER_SOL, Transaction } from "@solana/web3.js";
import { TransactionBuilder } from "@orca-so/common-sdk";
import { createTransferInstruction } from "spl-token";
import { Context, PDA, validateAndHashKycParams, Gender, RENEC_PROVIDER_TESTNET } from "..";
import { IsKycData, ProviderConfigData, UserConfigData, UserKycData } from "../types";

export class ProviderClient {
  ctx: Context;
  public pda: PDA;
  public nameHashRound: number;
  public documentIdHashRound: number;
  public dobHashRound: number;
  public doeHashRound: number;
  public genderHashRound: number;

  constructor(
    ctx: Context,
    pda: PDA,
    nameHashRound: number,
    documentIdHashRound: number,
    dobHashRound: number,
    doeHashRound: number,
    genderHashRound: number,
  ) {
    this.ctx = ctx;
    this.pda = pda;
    this.nameHashRound = nameHashRound;
    this.documentIdHashRound = documentIdHashRound;
    this.dobHashRound = dobHashRound;
    this.doeHashRound = doeHashRound;
    this.genderHashRound = genderHashRound;
  }

  public static async getClient(
    ctx: Context,
    nameHashRound: number = 1000,
    documentIdHashRound: number = 1000,
    dobHashRound: number = 1000,
    doeHashRound: number = 1000,
    genderHashRound: number = 1000,
  ): Promise<ProviderClient> {
    const pda = new PDA(ctx.program.programId);
    return new ProviderClient(
      ctx,
      pda,
      nameHashRound,
      documentIdHashRound,
      dobHashRound,
      doeHashRound,
      genderHashRound,
    );
  }

  public async submitKyc(
    providerName: string,
    user: PublicKey,
    salt: string,
    kycLevel: number,
    name: string,
    documentId: string,
    country: string,
    dob: Date,
    doe: Date | null,
    gender: Gender | null,
    isExpired: boolean,
  ): Promise<TransactionBuilder> {
    const providerConfig = this.pda.provider_config(providerName);
    const providerConfigData = await this.getOneProviderConfig(providerConfig.key);
    if (providerConfigData.provider.toString() !== this.ctx.wallet.publicKey.toBase58()) {
      throw Error("You are not the owner of this Provider Name");
    }
    const _isKyc = await this.isKyc(user);
    if (_isKyc.isKyc) {
      throw Error("User is already have KYC");
    }
    const userConfig = this.pda.user_config(user);
    const userKyc = this.pda.user_kyc(userConfig.key, SystemProgram.programId);
    const validatedParams = validateAndHashKycParams(
      this,
      salt,
      name,
      documentId,
      country,
      dob,
      doe,
      gender,
    );

    const tx = (
      await this.ctx.methods.submitKyc({
        accounts: {
          authority: this.ctx.wallet.publicKey,
          providerConfigAccount: providerConfig.key,
          userConfigAccount: userConfig.key,
          userKycAccount: userKyc.key,
        },
        inputs: {
          user,
          salt,
          kycLevel,
          name: validatedParams.encodedName,
          documentId: validatedParams.encodedDocumentId,
          country: validatedParams.encodedCountry,
          dob: validatedParams.encodedDob,
          doe: validatedParams.encodedDoe,
          gender: validatedParams.encodedGender,
          isExpired,
        },
      })
    ).toTx();

    return tx;
  }

  public async updateKyc(
    providerName: string,
    user: PublicKey,
    name: string,
    documentId: string,
    country: string,
    dob: Date,
    doe: Date | null,
    gender: Gender | null,
    isExpired: boolean,
  ): Promise<TransactionBuilder> {
    const providerConfig = this.pda.provider_config(providerName);
    const providerConfigData = await this.getOneProviderConfig(providerConfig.key);
    if (providerConfigData.provider.toString() !== this.ctx.wallet.publicKey.toBase58()) {
      throw Error("You are not the owner of this Provider Name");
    }
    const userConfig = this.pda.user_config(user);
    const userConfigData = await this.ctx.program.account.userConfig.fetch(userConfig.key);
    const newUserKyc = this.pda.user_kyc(userConfig.key, userConfigData.latestKycAccount);
    const validatedParams = validateAndHashKycParams(
      this,
      userConfigData.salt,
      name,
      documentId,
      country,
      dob,
      doe,
      gender,
    );

    const tx = (
      await this.ctx.methods.updateKyc({
        accounts: {
          authority: this.ctx.wallet.publicKey,
          providerConfigAccount: providerConfig.key,
          userConfigAccount: userConfig.key,
          oldUserKycAccount: userConfigData.latestKycAccount,
          newUserKycAccount: newUserKyc.key,
        },
        inputs: {
          name: validatedParams.encodedName,
          documentId: validatedParams.encodedDocumentId,
          country: validatedParams.encodedCountry,
          dob: validatedParams.encodedDob,
          doe: validatedParams.encodedDoe,
          gender: validatedParams.encodedGender,
          isExpired,
        },
      })
    ).toTx();

    return tx;
  }

  public async transferKycToken(user: PublicKey, mint: PublicKey): Promise<string> {
    // Get provider token account
    const providerTokenAccount = await this.ctx.connection.getTokenAccountsByOwner(
      this.ctx.wallet.publicKey,
      {
        mint,
      },
    );

    // Get user token account
    const userTokenAccount = await this.ctx.connection.getTokenAccountsByOwner(user, {
      mint,
    });

    let tx = new Transaction();
    tx.add(
      createTransferInstruction(
        providerTokenAccount.value[0].pubkey,
        userTokenAccount.value[0].pubkey,
        this.ctx.wallet.publicKey,
        LAMPORTS_PER_SOL,
      ),
    );
    const latestBlockHash = await this.ctx.connection.getLatestBlockhash("confirmed");
    tx.recentBlockhash = latestBlockHash.blockhash;
    tx.feePayer = this.ctx.wallet.publicKey;
    tx = await this.ctx.wallet.signTransaction(tx);

    return await this.ctx.provider.sendAndConfirm(tx);
  }

  public async getProviderConfigByProviderName(providerName: string): Promise<ProviderConfigData> {
    const pda = new PDA(this.ctx.program.programId);
    const providerConfig = pda.provider_config(providerName);
    return this.getOneProviderConfig(providerConfig.key);
  }

  public async getOneProviderConfig(providerConfig: PublicKey): Promise<ProviderConfigData> {
    const providerConfigData = await this.ctx.fetcher.getProviderConfig(providerConfig, true);
    if (!providerConfigData) {
      throw new Error(`Provider Config ${providerConfigData} not found`);
    }
    return providerConfigData;
  }

  public async getUserConfig(user: PublicKey): Promise<UserConfigData> {
    const pda = new PDA(this.ctx.program.programId);
    const userConfig = pda.user_config(user);

    const userConfigData = await this.ctx.fetcher.getUserConfig(userConfig.key, true);
    if (!userConfigData) {
      throw new Error(`User Config of user ${user} not found`);
    }
    return userConfigData;
  }

  public async getOneUserConfig(userConfig: PublicKey): Promise<UserConfigData> {
    const userConfigData = await this.ctx.fetcher.getUserConfig(userConfig, true);
    if (!userConfigData) {
      throw new Error(`User Config ${userConfig} not found`);
    }
    return userConfigData;
  }

  public async getCurrentUserKyc(user: PublicKey): Promise<UserKycData> {
    const pda = new PDA(this.ctx.program.programId);
    const userConfig = pda.user_config(user);

    const userConfigData = await this.ctx.fetcher.getUserConfig(userConfig.key, true);
    if (!userConfigData) {
      throw new Error(`User Config of user ${user} not found`);
    }

    return this.getOneUserKyc(userConfigData.latestKycAccount);
  }

  public async getOneUserKyc(userKyc: PublicKey): Promise<UserKycData> {
    const userKycData = await this.ctx.fetcher.getOneUserKyc(userKyc, true);
    if (!userKycData) {
      throw new Error(`User Kyc ${userKyc} not found`);
    }

    const userConfigData = await this.getOneUserConfig(userKycData.userConfigAccount);
    if (userConfigData.provider.toBase58() === RENEC_PROVIDER_TESTNET) {
      userKycData.provider = "RENEC blockchain";
    } else {
      userKycData.provider = userConfigData.provider.toBase58();
    }

    return userKycData;
  }

  public async isKyc(user: PublicKey): Promise<IsKycData> {
    const pda = new PDA(this.ctx.program.programId);
    const userConfig = pda.user_config(user);

    const userConfigData = await this.ctx.fetcher.getUserConfig(userConfig.key, true);
    if (!userConfigData) {
      return {
        isKyc: false,
        kycLevel: null,
        isExpired: null,
      };
    }

    const userKycData = await this.getOneUserKyc(userConfigData.latestKycAccount);
    return {
      isKyc: true,
      kycLevel: userConfigData.kycLevel,
      isExpired: userKycData.isExpired,
    };
  }
}

==> kyc-sdk-0.1.1.tgz: package/src/clients/provider-client.ts <==
import { PublicKey, SystemProgram, LAMPORTS_PER_SOL, Transaction } from "@solana/web3.js";
import { TransactionBuilder } from "@orca-so/common-sdk";
import { createTransferInstruction } from "spl-token";
import { Context, PDA, validateAndHashKycParams, Gender, RENEC_PROVIDER_MAINNET } from "..";
import { IsKycData, ProviderConfigData, UserConfigData, UserKycData } from "../types";

export class ProviderClient {
  ctx: Context;
  public pda: PDA;
  public nameHashRound: number;
  public documentIdHashRound: number;
  public dobHashRound: number;
  public doeHashRound: number;
  public genderHashRound: number;

  constructor(
    ctx: Context,
    pda: PDA,
    nameHashRound: number,
    documentIdHashRound: number,
    dobHashRound: number,
    doeHashRound: number,
    genderHashRound: number,
  ) {
    this.ctx = ctx;
    this.pda = pda;
    this.nameHashRound = nameHashRound;
    this.documentIdHashRound = documentIdHashRound;
    this.dobHashRound = dobHashRound;
    this.doeHashRound = doeHashRound;
    this.genderHashRound = genderHashRound;
  }

  public static async getClient(
    ctx: Context,
    nameHashRound: number = 1000,
    documentIdHashRound: number = 1000,
    dobHashRound: number = 1000,
    doeHashRound: number = 1000,
    genderHashRound: number = 1000,
  ): Promise<ProviderClient> {
    const pda = new PDA(ctx.program.programId);
    return new ProviderClient(
      ctx,
      pda,
      nameHashRound,
      documentIdHashRound,
      dobHashRound,
      doeHashRound,
      genderHashRound,
    );
  }

  public async submitKyc(
    providerName: string,
    user: PublicKey,
    salt: string,
    kycLevel: number,
    name: string,
    documentId: string,
    country: string,
    dob: Date,
    doe: Date | null,
    gender: Gender | null,
    isExpired: boolean,
  ): Promise<TransactionBuilder> {
    const providerConfig = this.pda.provider_config(providerName);
    const providerConfigData = await this.getOneProviderConfig(providerConfig.key);
    if (providerConfigData.provider.toString() !== this.ctx.wallet.publicKey.toBase58()) {
      throw Error("You are not the owner of this Provider Name");
    }
    const _isKyc = await this.isKyc(user);
    if (_isKyc.isKyc) {
      throw Error("User is already have KYC");
    }
    const userConfig = this.pda.user_config(user);
    const userKyc = this.pda.user_kyc(userConfig.key, SystemProgram.programId);
    const validatedParams = validateAndHashKycParams(
      this,
      salt,
      name,
      documentId,
      country,
      dob,
      doe,
      gender,
    );

    const tx = (
      await this.ctx.methods.submitKyc({
        accounts: {
          authority: this.ctx.wallet.publicKey,
          providerConfigAccount: providerConfig.key,
          userConfigAccount: userConfig.key,
          userKycAccount: userKyc.key,
        },
        inputs: {
          user,
          salt,
          kycLevel,
          name: validatedParams.encodedName,
          documentId: validatedParams.encodedDocumentId,
          country: validatedParams.encodedCountry,
          dob: validatedParams.encodedDob,
          doe: validatedParams.encodedDoe,
          gender: validatedParams.encodedGender,
          isExpired,
        },
      })
    ).toTx();

    return tx;
  }

  public async updateKyc(
    providerName: string,
    user: PublicKey,
    name: string,
    documentId: string,
    country: string,
    dob: Date,
    doe: Date | null,
    gender: Gender | null,
    isExpired: boolean,
  ): Promise<TransactionBuilder> {
    const providerConfig = this.pda.provider_config(providerName);
    const providerConfigData = await this.getOneProviderConfig(providerConfig.key);
    if (providerConfigData.provider.toString() !== this.ctx.wallet.publicKey.toBase58()) {
      throw Error("You are not the owner of this Provider Name");
    }
    const userConfig = this.pda.user_config(user);
    const userConfigData = await this.ctx.program.account.userConfig.fetch(userConfig.key);
    const newUserKyc = this.pda.user_kyc(userConfig.key, userConfigData.latestKycAccount);
    const validatedParams = validateAndHashKycParams(
      this,
      userConfigData.salt,
      name,
      documentId,
      country,
      dob,
      doe,
      gender,
    );

    const tx = (
      await this.ctx.methods.updateKyc({
        accounts: {
          authority: this.ctx.wallet.publicKey,
          providerConfigAccount: providerConfig.key,
          userConfigAccount: userConfig.key,
          oldUserKycAccount: userConfigData.latestKycAccount,
          newUserKycAccount: newUserKyc.key,
        },
        inputs: {
          name: validatedParams.encodedName,
          documentId: validatedParams.encodedDocumentId,
          country: validatedParams.encodedCountry,
          dob: validatedParams.encodedDob,
          doe: validatedParams.encodedDoe,
          gender: validatedParams.encodedGender,
          isExpired,
        },
      })
    ).toTx();

    return tx;
  }

  public async transferKycToken(user: PublicKey, mint: PublicKey): Promise<string> {
    // Get provider token account
    const providerTokenAccount = await this.ctx.connection.getTokenAccountsByOwner(
      this.ctx.wallet.publicKey,
      {
        mint,
      },
    );

    // Get user token account
    const userTokenAccount = await this.ctx.connection.getTokenAccountsByOwner(user, {
      mint,
    });

    let tx = new Transaction();
    tx.add(
      createTransferInstruction(
        providerTokenAccount.value[0].pubkey,
        userTokenAccount.value[0].pubkey,
        this.ctx.wallet.publicKey,
        LAMPORTS_PER_SOL,
      ),
    );
    const latestBlockHash = await this.ctx.connection.getLatestBlockhash("confirmed");
    tx.recentBlockhash = latestBlockHash.blockhash;
    tx.feePayer = this.ctx.wallet.publicKey;
    tx = await this.ctx.wallet.signTransaction(tx);

    return await this.ctx.provider.sendAndConfirm(tx);
  }

  public async getProviderConfigByProviderName(providerName: string): Promise<ProviderConfigData> {
    const pda = new PDA(this.ctx.program.programId);
    const providerConfig = pda.provider_config(providerName);
    return this.getOneProviderConfig(providerConfig.key);
  }

  public async getOneProviderConfig(providerConfig: PublicKey): Promise<ProviderConfigData> {
    const providerConfigData = await this.ctx.fetcher.getProviderConfig(providerConfig, true);
    if (!providerConfigData) {
      throw new Error(`Provider Config ${providerConfigData} not found`);
    }
    return providerConfigData;
  }

  public async getUserConfig(user: PublicKey): Promise<UserConfigData> {
    const pda = new PDA(this.ctx.program.programId);
    const userConfig = pda.user_config(user);

    const userConfigData = await this.ctx.fetcher.getUserConfig(userConfig.key, true);
    if (!userConfigData) {
      throw new Error(`User Config of user ${user} not found`);
    }
    return userConfigData;
  }

  public async getOneUserConfig(userConfig: PublicKey): Promise<UserConfigData> {
    const userConfigData = await this.ctx.fetcher.getUserConfig(userConfig, true);
    if (!userConfigData) {
      throw new Error(`User Config ${userConfig} not found`);
    }
    return userConfigData;
  }

  public async getCurrentUserKyc(user: PublicKey): Promise<UserKycData> {
    const pda = new PDA(this.ctx.program.programId);
    const userConfig = pda.user_config(user);

    const userConfigData = await this.ctx.fetcher.getUserConfig(userConfig.key, true);
    if (!userConfigData) {
      throw new Error(`User Config of user ${user} not found`);
    }

    return this.getOneUserKyc(userConfigData.latestKycAccount);
  }

  public async getOneUserKyc(userKyc: PublicKey): Promise<UserKycData> {
    const userKycData = await this.ctx.fetcher.getOneUserKyc(userKyc, true);
    if (!userKycData) {
      throw new Error(`User Kyc ${userKyc} not found`);
    }

    const userConfigData = await this.getOneUserConfig(userKycData.userConfigAccount);
    if (userConfigData.provider.toBase58() === RENEC_PROVIDER_MAINNET) {
      userKycData.provider = "RENEC blockchain";
    } else {
      userKycData.provider = userConfigData.provider.toBase58();
    }

    return userKycData;
  }

  public async isKyc(user: PublicKey): Promise<IsKycData> {
    const pda = new PDA(this.ctx.program.programId);
    const userConfig = pda.user_config(user);

    const userConfigData = await this.ctx.fetcher.getUserConfig(userConfig.key, true);
    if (!userConfigData) {
      return {
        isKyc: false,
        kycLevel: null,
        isExpired: null,
      };
    }

    const userKycData = await this.getOneUserKyc(userConfigData.latestKycAccount);
    return {
      isKyc: true,
      kycLevel: userConfigData.kycLevel,
      isExpired: userKycData.isExpired,
    };
  }
}
